# coding:utf-8
import weakref

from qtpy.QtCore import QObject, Signal
from qtpy.QtWidgets import QWidget


class Proxy(QObject):

    valueChanged = Signal()

    def __init__(self):
        super().__init__()

    @property
    def value(self):
        return None

    def __repr__(self):
        return str(self.value)

    def __eq__(self, other):
        return self.value == other

    def __ne__(self, other):
        return self.value != other

    def __add__(self, other):
        return self.value + other

    def __radd__(self, other):
        return self + other

    def __iadd__(self, other):
        self.value += other
        return self

    def __sub__(self, other):
        return self.value - other

    def __isub__(self, other):
        self.value -= other
        return self

    def __rsub__(self, other):
        return other - self.value

    def __mul__(self, other):
        return self.value * other

    def __rmul__(self, other):
        return other * self.value

    def __imul__(self, other):
        self.value *= other
        return self

    def __mod__(self, other):
        return self.value % other

    def __rmod__(self, other):
        return other % self.value

    def __imod__(self, other):
        self.value %= other
        return self

    def __pow__(self, other):
        return self.value ** other

    def __rpow__(self, other):
        return other ** self.value

    def __ipow__(self, other):
        self.value **= other
        return self

    def __truediv__(self, other):
        return self.value / other

    def __floordiv__(self, other):
        return self.value // other

    def __gt__(self, other):
        return self.value > other

    def __lt__(self, other):
        return self.value < other

    def __ge__(self, other):
        return self.value >= other

    def __le__(self, other):
        return self.value <= other

    def __bool__(self):
        return bool(self.value)

    def __neg__(self):
        return -self.value

    def __abs__(self):
        return abs(self.value)

    def __pos__(self):
        return self.value

    def __len__(self):
        return len(self.value)

    def __or__(self, other):
        return self.value | other

    def __and__(self, other):
        return self.value & other

    def __rand__(self, other):
        return other & self.value

    def __iand__(self, other):
        self.value &= other
        return self

    def __xor__(self, other):
        return self.value ^ other

    def __rxor__(self, other):
        return other ^ self.value

    def __ixor__(self, other):
        self.value ^= other
        return self

    def __ror__(self, other):
        return other | self.value

    def __ior__(self, other):
        self.value |= other
        return self

    def __lshift__(self, other):
        return self.value << other

    def __rlshift__(self, other):
        return other << self.value

    def __ilshift__(self, other):
        self.value <<= other
        return self

    def __rshift__(self, other):
        return self.value >> other

    def __rrshift__(self, other):
        return other >> self.value

    def __irshift__(self, other):
        self.value >>= other
        return self

    def __round__(self, n=None):
        return round(self.value, n) if n is not None else round(self.value)

    def __int__(self):
        return int(self.value)

    def __float__(self):
        return float(self.value)

    def __complex__(self):
        return complex(self.value)

    def __index__(self):
        return int(self.value)

    def __invert__(self):
        return ~self.value



# coding:utf-8
import weakref

from qtpy.QtCore import QObject, Signal
from qtpy.QtWidgets import QWidget

from .proxy import Proxy


class Ref(Proxy):

    def __init__(self, widget: QWidget, property: str):
        super().__init__()
        self.watchedProperty = property
        self.widgetRef = weakref.ref(widget)

    @property
    def widget(self):
        return self.widgetRef()

    @property
    def value(self):
        if self.widget is None:
            return None

        return self.widget.property(self.watchedProperty)

    @value.setter
    def value(self, value):
        if value == self.value:
            return

        self.widget.setProperty(self.watchedProperty, value)
        self.valueChanged.emit()

    def __repr__(self):
        return str(self.value)

    def __eq__(self, other):
        return self.value == other

    def __ne__(self, other):
        return self.value != other

    def __add__(self, other):
        return self.value + other

    def __radd__(self, other):
        return self + other

    def __iadd__(self, other):
        self.value += other
        return self

    def __sub__(self, other):
        return self.value - other

    def __isub__(self, other):
        self.value -= other
        return self

    def __rsub__(self, other):
        return other - self.value

    def __mul__(self, other):
        return self.value * other

    def __truediv__(self, other):
        return self.value / other

    def __rtruediv__(self, other):
        return other / self.value

    def __itruediv__(self, other):
        self.value /= other
        return self

    def __floordiv__(self, other):
        return self.value // other

    def __rfloordiv__(self, other):
        return other // self.value

    def __ifloordiv__(self, other):
        self.value //= other
        return self

    def __gt__(self, other):
        return self.value > other

    def __lt__(self, other):
        return self.value < other

    def __ge__(self, other):
        return self.value >= other

    def __le__(self, other):
        return self.value <= other

    def __bool__(self):
        return bool(self.value)

    def __neg__(self):
        return -self.value

    def __abs__(self):
        return abs(self.value)

    def __pos__(self):
        return self.value


# coding: utf-8
from typing import Union
from qtpy.QtWidgets import QWidget

from .ref import Ref
from .computed import Computed


class QMVVM:

    def __init__(self, *args, **kwargs):
        pass

    def bind(self, value, widget: QWidget, property="text") -> Union[Ref, Computed]:
        if callable(value):
            reactive = Computed(widget, property, value)
            reactive.update()
        else:
            reactive = Ref(widget, property)
            reactive.value = value

        return reactive



import setuptools


with open('README.md', encoding='utf-8') as f:
    long_description = f.read()

setuptools.setup(
    name="QMVVM",
    version="0.0.2",
    keywords="qt pyqt pyside mvvm",
    author="zhiyiYo",
    author_email="shokokawaii@outlook.com",
    description="A Model-View-ViewModel(MVVM) framework for Qt",
    long_description=long_description,
    long_description_content_type='text/markdown',
    license="Apache-2.0",
    url="https://github.com/zhiyiYo/QMVVM",
    packages=setuptools.find_packages(),
    install_requires=[
        "qtpy",
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent'
    ],
    project_urls={
        'Documentation': 'https://github.com/zhiyiYo/QMVVM',
        'Source Code': 'https://github.com/zhiyiYo/QMVVM',
        'Bug Tracker': 'https://github.com/zhiyiYo/QMVVM/issues',
    }
)
